# -*- coding: utf-8 -*-
"""
Project templates
=================

Available embedded project templates to use with project starter.

"""
TEMPLATE_ALIAS = {
    'basic': 'optimus.samples.basic',
    'i18n': 'optimus.samples.i18n',
}
