{
    "SOURCES_PATH": "sources/scss",
    "LIBRARY_PATHS": [],
    "TARGET_PATH": "sources/css",
    "OUTPUT_STYLES": "expanded",
    "SOURCE_COMMENTS": false,
    "SOURCE_MAP": false,
    "EXCLUDES": []
}
